import logbook
from . import hooks

_LOGGER_NAME = "slash.step"
_STEP_LOG_LEVEL = logbook.NOTICE

_logger = logbook.Logger(_LOGGER_NAME)


class Step:

    def __init__(self, msg, *args, **kwargs):
        super().__init__()
        self.message = msg.format(*args, **kwargs) if args or kwargs else msg

    def __str__(self):
        return self.message

    def __repr__(self):
        return f"<Step {self.message!r}>"

    def _start(self):
        _logger.log(_STEP_LOG_LEVEL, self.message)
        hooks.step_start.trigger({})

    def _success(self):
        hooks.step_success.trigger({})

    def _error(self):
        hooks.step_error.trigger({})

    def _end(self):
        hooks.step_end.trigger({})

    def __enter__(self):
        self._start()
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        try:
            if exc_type is None:
                self._success()
            else:
                self._error()
        finally:
            self._end()



import gossip
import logbook
import pytest
import uuid
from munch import Munch


@pytest.fixture(scope="session", autouse=True)
def setup_logging():
    logbook.compat.redirect_logging()
    logbook.StderrHandler().push_application()


@pytest.fixture
def called_counters(request):
    obj = CalledCounters()
    request.addfinalizer(obj.unregister)
    return obj


class CalledCounters:
    def __init__(self):
        super().__init__()
        self._called = Munch(start=0, error=0, end=0, success=0)
        self._token = str(uuid.uuid4())
        self._is_active = True
        gossip.register(self._on_step_start, 'slash.step_start', token=self._token)
        gossip.register(self._on_step_end, 'slash.step_end', token=self._token)
        gossip.register(self._on_step_success, 'slash.step_success', token=self._token)
        gossip.register(self._on_step_error, 'slash.step_error', token=self._token)

    def unregister(self):
        gossip.unregister_token(self._token)
        self._is_active = False

    def _on_step_start(self):
        self._called.start += 1

    def _on_step_error(self):
        self._called.error += 1

    def _on_step_end(self):
        self._called.end += 1

    def _on_step_success(self):
        self._called.success += 1

    def verify_calls(self, start=1, success=1, end=1, error=0):
        assert self._is_active
        assert self._called == dict(start=start, success=success, end=end, error=error)



# pylint: disable=unused-import
from .__version__ import __version__
from . import hooks
from .step import Step, _LOGGER_NAME, _STEP_LOG_LEVEL
import slash.log

slash.log.set_log_color(_LOGGER_NAME, _STEP_LOG_LEVEL, "blue")

STEP = Step



import gossip

step_hook_group = gossip.create_group("step")
step_hook_group.set_strict()

step_start = gossip.define('slash.step_start')
step_success = gossip.define('slash.step_success')
step_error = gossip.define('slash.step_error')
step_end = gossip.define('slash.step_end')



#!/usr/bin/env python
from setuptools import setup


setup(
    setup_requires=['pbr>=3.0', 'setuptools>=17.1'],
    pbr=True,
    python_requires=">=3.6.*",
    long_description_content_type='text/markdown; charset=UTF-8',
)



import pytest
from slash_step import STEP


def test_step_entry():
    msg = "Some step message"
    with STEP(msg) as step:
        assert isinstance(step, STEP)
        assert str(step) == msg
        assert msg in repr(step)


def test_step_success(called_counters):
    called_counters.verify_calls(start=0, success=0, end=0, error=0)
    with STEP("This will succeed"):
        called_counters.verify_calls(start=1, success=0, end=0, error=0)
    called_counters.verify_calls(start=1, success=1, end=1, error=0)


class CustomError(Exception):
    pass


def test_step_error(called_counters):
    called_counters.verify_calls(start=0, success=0, end=0, error=0)
    with pytest.raises(CustomError):
        with STEP("This will fail"):
            called_counters.verify_calls(start=1, success=0, end=0, error=0)
            raise CustomError('My exception')
    called_counters.verify_calls(start=1, success=0, end=1, error=1)


def test_step_creation_with_arguments():
    step = STEP("Message with args and kwargs", "args", kwargs='kwargs')
    assert step.message, "Message with args and kwargs"


def test_step_creation_with_curly_brackets_with_arguments():
    step = STEP("Message with {} and {kwargs}", "args", kwargs="some_kwargs")
    assert step.message, "Message with args and some_kwargs"


def test_step_creation_with_curly_brackets_without_arguments():
    message = "My message with {curly brackets}"
    step = STEP(message)
    assert step.message, message


def test_nested_steps(called_counters):
    with STEP("Outer step"):
        called_counters.verify_calls(start=1, success=0, end=0, error=0)
        with pytest.raises(CustomError):
            with STEP("Inner step"):
                called_counters.verify_calls(start=2, success=0, end=0, error=0)
                raise CustomError('My exception')
        called_counters.verify_calls(start=2, success=0, end=1, error=1)
    called_counters.verify_calls(start=2, success=1, end=2, error=1)
